/**
 * Copyright (C) 2020 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * See about document.execCommand: https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { RE.backuprange(); });

// Initializations
RE.callback = function() {
    window.location.href = "re-callback://" + encodeURIComponent(RE.getHtml());
    setTimeout(function() {
               RE.enabledEditingItems();
           },50)
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
    RE.enabledEditingItems()
}

RE.redo = function() {
    document.execCommand('redo', false, null);
    RE.enabledEditingItems()
}


RE.setBold = function() {
    document.execCommand('bold', false, null);
     RE.enabledEditingItems()
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
     RE.enabledEditingItems()
}

RE.setBullets = function() {
    document.execCommand('insertUnorderedList', false, null);
     RE.enabledEditingItems()
}

RE.setNumbers = function() {
    document.execCommand('insertOrderedList', false, null);
     RE.enabledEditingItems()
}

RE.setTextColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}




RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
}

RE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
}

//引用整篇文章(用户没有专业认证标签)
RE.quoteWholeForNoLabel = function(id, img, name, score, job, type, content) {
    var html ='<div><br></div><a class="queteWhole" contenteditable="true" style="text-decoration:none;color:#000;" href="##'+id+"@wholeText"+ '"><div contenteditable="false" style="-webkit-user-select: none;background: rgb(240, 244, 247); border-radius: 6px; padding: 12px 0px;position:relative;"><div style="border-left: 2px solid rgb(30, 151, 241); padding: 0px 12px;"><div style="display: flex; text-decoration: none; color: rgb(0, 0, 0); position: relative; margin: 0px 0px 12px; border-bottom: 1px solid rgb(230, 230, 230); padding: 0px 0px 12px;"><span style="display: block; width: 40px; height: 40px; margin-right: 5px; background: url(' + img + ') center center / 44px 44px; border-radius: 50%;"></span><span style="font-size: 10px; color: #999;"><span style="font-size: 14px; color: rgb(51, 51, 51);font-weight: bold;margin-right: 4px;height: 20px;line-height: 20px;">' + name + '</span><span style="height: 14px;vertical-align: bottom;font-size: 10px;box-sizing: border-box;"> 认证积分:' + score + '</span><p style="margin: 0; color:#999; border-radius: 2px; padding:2px 2px 1px 2px; margin-top: 6px;width:230px;font-size: 10px;overflow: hidden;text-overflow:ellipsis;white-space: nowrap;">'+job+'</p></span></div><span class="gezhi" style="word-break:break-all;font-size: 16px; font-weight: bold; color: rgb(51, 51, 51);vertical-align: middle;overflow : hidden;text-overflow: ellipsis;display: -webkit-box;-webkit-line-clamp: 4;-webkit-box-orient: vertical;"><span style="position:relative;display:inline-block;width:22px;height:14px;border: 1px solid rgb(30, 151, 241); color: rgb(30, 151, 241); border-radius: 2px; margin-right: 5px; font-size: 9px;vertical-align: inherit;box-sizing:border-box;" ><span style="position:absolute;top:50%;left:50%;transform:translate(-50%,-49%);width:20px;text-align:center;">' + type + '</span></span>'+ content + '</span></div></div></a><div><br></div><div><br></div>';
    RE.insertHTML(html);
}

//引用整篇文章
RE.quoteWhole = function(id, img, name, score, job, type, content) {
    var html ='<div><br></div><a class = "queteWhole" contenteditable="true" style="text-decoration:none;color:#000;" href="##'+id+"@wholeText"+ '"><div contenteditable="false" style="-webkit-user-select: none;background: rgb(240, 244, 247); border-radius: 6px; padding: 12px 0px;position:relative;"><div  style="border-left: 2px solid rgb(30, 151, 241); padding: 0px 12px;"><div style="display: flex; text-decoration: none; color: rgb(0, 0, 0); position: relative; margin: 0px 0px 12px; border-bottom: 1px solid rgb(230, 230, 230); padding: 0px 0px 12px;"><span  style="display: block; width: 40px; height: 40px; margin-right: 5px; background: url(' + img + ') center center / 40px 40px; border-radius: 50%;"></span><span style="font-size: 11px; color: color: #999;"><span style="font-size: 14px; color: rgb(51, 51, 51);font-weight: bold;margin-right: 4px;height: 20px;line-height: 20px;">' + name + '</span><span style="height: 14px;vertical-align: bottom;font-size: 10px;box-sizing: border-box;"> 认证积分:' + score + '</span><p style="margin: 0; background: rgb(208, 229, 245); color: rgb(47, 135, 203); border-radius: 2px; padding:2px 2px 1px 2px; margin-top: 6px;width:fit-content;font-size: 10px;">' + job + ' </p></span></div><span class="gezhi" style="word-break:break-all;font-size: 16px; font-weight: bold; color: rgb(51, 51, 51);vertical-align: middle;overflow : hidden;text-overflow: ellipsis;display: -webkit-box;-webkit-line-clamp: 4;-webkit-box-orient: vertical;"><span style="position:relative;display:inline-block;width:22px;height:14px;border: 1px solid rgb(30, 151, 241); color: rgb(30, 151, 241); border-radius: 2px; margin-right: 5px; font-size: 9px;vertical-align: inherit;box-sizing:border-box;" ><span style="position:absolute;top:50%;left:50%;transform:translate(-50%,-49%);width:20px;text-align:center;">' + type + '</span></span>' + content + '</span></div></div></a><div><br></div><div><br></div>';
    RE.insertHTML(html);
}

//引用文章段落
RE.quotePart = function(id, title, content) {
    var html ='<div><br></div><a class = "quetePart" contenteditable="true" onselectstart="return false" style="text-decoration:none;color:#000;" href="##'+id+"@partText"+ "@@"+ content +"!!"+'"><div  contenteditable="false" style="-webkit-user-select: none; -moz-user-select: none;-o-user-select: none;user-select: none;border-radius: 6px;background:#F0F4F7;padding:12px 0;position:relative;"><div class="gezhi" style="border-left:2px solid #1E97F1;padding:0 12px;word-break:break-all;">' + content + '</div></div></a><div><br></div><div><br></div>';
    RE.insertHTML(html);
}

//插入图片
RE.insertImage = function(url, alt) {
    var html='<div><br></div><div class="gezhi_i" contenteditable="true" ><div  contenteditable="false" style="-webkit-user-select: none;position:relative;"><img class = "img_main" style="border-radius: 4px;width:100%" src="' + url + '" alt="' + alt + '" /><a class = "img_cancel" style="display:block;width:22px;height:22px;position:absolute;top:14px;left:8px;background:url(./image_delete.png);background-size:22px;z-index:999;"  href="image_delete##'+url+'" onclick="return false"></a></div></div><br><br>'
    RE.insertHTML(html);
}

//获取所有引证id
RE.getAllQuoteIds = function() {
     var result = document.getElementsByTagName("a");
     var array=Array.from(result);
        var quoteIds="";
        array.forEach((item,index) => {
                quoteIds+="gezhi_quote"+decodeURI(item.href);
              });
         return quoteIds;
}

//获取所有引证数据(即引证内容)
RE.getAllQuote = function() {
    var result = document.getElementsByClassName("gezhi");
    var array=Array.from(result);
    var quotes="";
    array.forEach((item,index) => {
            quotes+="gezhi_quote"+item.innerHTML;
          });
     return quotes;
}

//获取所有引证数据(即引证内容，发布追加角标用)
RE.getAllQuoteForAppendSup = function() {
    var result = document.getElementsByClassName("gezhi");
    var array=Array.from(result);
    var quotes="";
    array.forEach((item,index) => {
                console.log("item.clientHeight："+item.clientHeight);
                 console.log("item.scrollHeight："+item.scrollHeight);
                 console.log("item.style："+item.style);
         if (item.clientHeight >=item.scrollHeight  ) {
            item.style.display="inline";
         }
         quotes+="gezhi_quote"+item.outerHTML;
          });
     return quotes;
}

//获取所有引证完整数据
RE.getAllQuotes = function() {
     var result = document.getElementsByTagName("a");
     var array=Array.from(result);
     console.log("++++++++++++++++++++++++++++"+array);
        var quoteIds="";
        array.forEach((item,index) => {
                quoteIds+="gezhi_quote"+decodeURIComponent(item.outerHTML.replace(/\%/g, '%25'));
              });
         return quoteIds;
}

//获取所有图片数据
RE.getImages = function() {
    var result = document.getElementsByClassName("gezhi_i");
    var array=Array.from(result);
    var quotes="";
    array.forEach((item,index) => {
            quotes+="gezhi_image"+item.innerHTML;
          });
     return quotes;
}

//获取所有链接数据
RE.getLinks = function() {
    var result = document.getElementsByClassName("gezhi_l");
    var array=Array.from(result);
    var quotes="";
    array.forEach((item,index) => {
            quotes+="gezhi_links"+item.outerHTML;
          });
     return quotes;
}

//获取所有引证角标
RE.getQuoteSups = function() {
    var result = document.getElementsByTagName("sup");
    var array=Array.from(result);
    var quoteIds="";
    array.forEach((item,index) => {
         quoteIds+="gezhi_quote"+item.outerHTML;
     });
     return quoteIds;
}

//获取引证数量
RE.getQuotesNum = function() {
    var result = document.getElementsByClassName("gezhi");
    var array=Array.from(result);
    return array.length;
}

//获取论述字数
RE.getTextLength = function() {
    return RE.editor.innerText.length;
}

//获取链接数量
RE.getLinksNum = function() {
    var result = document.getElementsByClassName("gezhi_l");
    var array=Array.from(result);
    return array.length;
}

RE.insertImageW = function(url, alt, width) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '"/>';
    RE.insertHTML(html);
}

RE.insertImageWH = function(url, alt, width, height) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '" height="' + height +'"/>';
    RE.insertHTML(html);
}

RE.insertVideo = function(url, alt) {
    var html = '<video src="' + url + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoW = function(url, width) {
    var html = '<video src="' + url + '" width="' + width + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoWH = function(url, width, height) {
    var html = '<video src="' + url + '" width="' + width + '" height="' + height + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertAudio = function(url, alt) {
    var html = '<audio src="' + url + '" controls></audio><br>';
    RE.insertHTML(html);
}

RE.insertYoutubeVideo = function(url) {
    var html = '<iframe width="100%" height="100%" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoW = function(url, width) {
    var html = '<iframe width="' + width + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoWH = function(url, width, height) {
    var html = '<iframe width="' + width + '" height="' + height + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

//插入外链
RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        var html ='<div><br></div><span class="gezhi_l" contenteditable="false" style="-webkit-user-select: none;display: flex;align-items: center;"><span  style="display:inline-block;width:14px;height:14px;margin-right:2px;background:url(https://gezhiapi.oss-cn-shanghai.aliyuncs.com/user/img/Lark20210512-100308.png) center;background-size:14px 14px;"></span><a style="text-decoration:none;color:#1E97F1;font-size:16px;“ contenteditable="false" onclick="return false;" href="'+url+'">'+title+'</a></span><div><br></div><div><br></div>';
        RE.insertHTML(html);
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
       RE.callback();
   }
}

RE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}



RE.enabledEditingItems = function(e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    window.location.href = "re-state://" + encodeURI(items.join(','));
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
//    var selection = window.getSelection();
//    selection.removeAllRanges();
//    selection.addRange(range);
    RE.editor.focus();
}

RE.focusEnd=function ()
           {
               var el=document.getElementById("editor");
               //el=el[0];  //jquery 对象转dom对象
               el.focus();

               if($.support.msie)
               {
                   var range = document.selection.createRange();
                   this.last = range;
                   range.moveToElementText(el);
                   range.select();
                   document.selection.empty(); //取消选中
               }
               else
               {
                   var range = document.createRange();
                   range.selectNodeContents(el);
                   range.collapse(false);
                   var sel = window.getSelection();
                   sel.removeAllRanges();
                   sel.addRange(range);
               }
           }

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}




// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
});

RE.editor.addEventListener("click", RE.enabledEditingItems);




